#version 120

varying vec3 fragPos;
varying vec3 Normal;

varying vec4 ShadowCoord;
uniform sampler2D ShadowMap;

uniform vec3 lightPos;
uniform vec3 viewPos;

//for PCF sampling
uniform vec2 texelSize;

float ShadowCalculation(vec4 shadowCoord) {
    vec4 shadowCoordinateWdivide = shadowCoord / shadowCoord.w ;
    
    // Used to lower moir pattern and self-shadowing
    vec3 normal = normalize(Normal);
    vec3 lightDir = normalize(lightPos - fragPos);
    float bias = max(0.03 * (1.0 - dot(normal, lightDir)), 0.003);
    shadowCoordinateWdivide.z -= bias;
    
    //with PCF
    float shadow = 0.0;
    float x,y;
    for (y = -4.5; y <= 4.5; y += 1.0) {
        for (x = -4.5; x <= 4.5; x += 1.0) {
            float pcfDepth = texture2D(ShadowMap, shadowCoordinateWdivide.xy + vec2(x, y) * texelSize).r;
            shadow += pcfDepth < shadowCoordinateWdivide.z ? 0.0 : 1.0;
        }
    }
    shadow /= 100.0;
    //without PCF
    /*
     float distanceFromLight = texture2D(ShadowMap,shadowCoordinateWdivide.st).z;
     float shadow = 1.0;
     if (shadowCoord.w > 0.0)
     //shadow = distanceFromLight < shadowCoordinateWdivide.z ? 0.5 : 1.0 ;
     shadow = distanceFromLight < shadowCoordinateWdivide.z ? 0.0 : 1.0 ;
     */
    
    return shadow;
}


void main()
{
    vec3 color = vec3(gl_Color);
    vec3 normal = normalize(Normal);
    vec3 lightColor = vec3(1.0);
    // Ambient
    vec3 ambient = 0.15 * color;
    // Diffuse
    vec3 lightDir = normalize(lightPos - fragPos);
    float diff = max(dot(lightDir, normal), 0.0);
    vec3 diffuse = diff * lightColor;
    // Specular
    vec3 viewDir = normalize(viewPos - fragPos);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = 0.0;
    vec3 halfwayDir = normalize(lightDir + viewDir);
    spec = pow(max(dot(normal, halfwayDir), 0.0), 64.0);
    vec3 specular = spec * lightColor;
    // Calculate shadow
    float shadow = ShadowCalculation(ShadowCoord);
    //vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * color;
    vec3 lighting = (ambient + (shadow) * (diffuse + specular)) * color;
    
    //gl_FragColor = shadow * gl_Color;
    gl_FragColor = vec4(lighting, 1.0);
}