varying vec3 N;
varying vec3 v;

uniform samplerCube cubemap;
uniform vec3 cameraPos;

#define MAX_LIGHTS 3

void main (void) {
    vec3 norm = normalize(N);
    vec4 finalColor = vec4(0.0, 0.0, 0.0, 0.0);
    
    for (int i=0;i<MAX_LIGHTS;i++)
    {
        vec3 L = normalize(gl_LightSource[i].position.xyz - v);
        vec3 E = normalize(-v); // we are in Eye Coordinates, so EyePos is (0,0,0)
        vec3 R = normalize(-reflect(L,norm));
        
        //calculate Ambient Term:
        vec4 Iamb = gl_FrontLightProduct[i].ambient;
        //calculate Diffuse Term:
        vec4 Idiff = gl_FrontLightProduct[i].diffuse * max(dot(norm,L), 0.0);
        Idiff = clamp(Idiff, 0.0, 1.0);
        
        // calculate Specular Term:
        vec4 Ispec = gl_FrontLightProduct[i].specular
        * pow(max(dot(R,E),0.0),0.3*gl_FrontMaterial.shininess);
        Ispec = clamp(Ispec, 0.0, 1.0);
        
        finalColor += Iamb + Idiff + Ispec;
    }
    
    vec3 ReflectDir = vec3(cameraPos);
    ReflectDir = normalize(reflect(ReflectDir,normalize(N)));
    // write Total Color:
    vec4 reflect = textureCube(cubemap, ReflectDir);
    clamp(reflect, 0.0, 1.0);
    finalColor += gl_FrontLightModelProduct.sceneColor;
    finalColor =  finalColor + (0.3 * reflect);
    clamp(finalColor, 0.0, 1.0);
    
    gl_FragColor = finalColor;
}